﻿namespace Kompozyt
{
    /*public interface IPracownik
    {
        public void WyświetlInformacje(int głębokość);
    }*/

    public interface IOdwiedzający
    {
        public void Odwiedź(Pracownik pracownik, int głębokość);
    }

    public class Pracownik //liść
    {
        private string imię, nazwisko, stanowisko;

        protected internal bool odwiedzony = false;

        public Pracownik(string imię, string nazwisko, string stanowisko)
        {
            this.imię = imię;
            this.nazwisko = nazwisko;
            this.stanowisko = stanowisko;
        }

        public override string ToString()
        {
            return $"{imię} {nazwisko} ({stanowisko})";
        }

        /*
        public virtual void WyświetlInformacje(int głębokość)
        {
            for (int i = 0; i < głębokość; ++i) Console.Write("  ");
            Console.WriteLine($"{imię} {nazwisko} ({stanowisko})");
            odwiedzony = true;
        }
        */

        public virtual void PrzyjmijWizytę(IOdwiedzający odwiedzający, int głębokość)
        {
            odwiedzający.Odwiedź(this, głębokość);
            odwiedzony = true;
        }

        public virtual void Resetuj()
        {
            odwiedzony = false;
        }
    }

    public class Kierownik : Pracownik //gałąź, węzeł --> kompozyt (umię poradzić sobie z cyklami)
    {
        private List<Pracownik> podwładni = new List<Pracownik>();

        public Kierownik(string imię, string nazwisko, string stanowisko)
            :base(imię, nazwisko, stanowisko)
        {
        }

        public void DodajPodwładnego(Pracownik pracownik)
        {
            if(!podwładni.Contains(pracownik))
                podwładni.Add(pracownik);
        }

        private static int? głębokośćNajwyższegoKierownika = null;

        /*
        public override void WyświetlInformacje(int głębokość = 0)
        {
            if (głębokośćNajwyższegoKierownika == null)
                głębokośćNajwyższegoKierownika = głębokość;

            base.WyświetlInformacje(głębokość);
            foreach (Pracownik pracownik in podwładni)
                if (!pracownik.odwiedzony)
                    pracownik.WyświetlInformacje(głębokość + 1);
                else Console.WriteLine("Cykl!");

            if (głębokość == głębokośćNajwyższegoKierownika) Resetuj();
        }
        */

        public override void PrzyjmijWizytę(IOdwiedzający odwiedzający, int głębokość = 0)
        {
            if (głębokośćNajwyższegoKierownika == null) głębokośćNajwyższegoKierownika = głębokość;

            base.PrzyjmijWizytę(odwiedzający, głębokość);
            foreach (Pracownik pracownik in podwładni)
                if (!pracownik.odwiedzony)
                    pracownik.PrzyjmijWizytę(odwiedzający, głębokość + 1);                

            if (głębokość == głębokośćNajwyższegoKierownika) Resetuj();
        }

        public override void Resetuj()
        {
            base.Resetuj();
            foreach (Pracownik pracownik in podwładni)                
                if(pracownik.odwiedzony) pracownik.Resetuj();
        }
    }

    public class OdwiedzającyWyświetlającyInformacje : IOdwiedzający
    {
        public void Odwiedź(Pracownik pracownik, int głębokość)
        {
            for (int i = 0; i < głębokość; ++i) Console.Write("  ");
            Console.WriteLine(pracownik);
        }
    }

    public class OdwiedzającySpłaszczającyDoListy : IOdwiedzający
    {
        private List<Pracownik> lista = new List<Pracownik>();

        public void Odwiedź(Pracownik pracownik, int głębokość)
        {
            lista.Add(pracownik);
        }

        public List<Pracownik> Lista { get { return lista; } }
    }

    public class OdwiedzającyZliczający : IOdwiedzający
    {
        public void Odwiedź(Pracownik pracownik, int głębokość)
        {
            throw new NotImplementedException();
        }
    }
}
